<?php

/**
 * GenericDb.php - PHPOF2 generic auditor for storing information in a database
 *
 * (c)2001-2008 Tim Jackson (tim@timj.co.uk)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of version 3 of the GNU Lesser General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package PHPOF2_Audit_GenericDb
 */

require_once 'PHPOF2/Audit/Interface.php';

/**
 * Generic database auditing class for PHPOF2
 *
 * This class is designed as a (working) example of how to use the PHPOF2 audit
 * capabilities. It logs audit changes to two tables in a relational database:
 *
 *  an "audit events" table, with details of the event
 *  an "audit changes" table, with details of the individual changes within that event
 *
 * It also supports logging of generic "notes" (which don't have any changes
 * associated with them but are relationally linked to some entity in the 
 * database) via the constant PHPOF2_Audit_GenericDb::EVENT_TYPE_NOTE.
 * 
 */
class PHPOF2_Audit_GenericDb implements PHPOF2_Audit_Interface
{

	private $_db = null;
	private $_event_table_name = 'audit_events';
	private $_change_table_name = 'audit_event_changes';
	
	/**
	 * Used for logging an audit event which is not actually a change to the
	 * database, but is merely a note about a database entity
	 */
	const EVENT_TYPE_NOTE = 'note';
	
	/**
	 * Initialise the class.
	 *
	 * @param string|MDB2_Driver_Common $dsn      Either an MDB2-compatible DSN to connect to, or an instantiated MDB2_Driver_Common object (such as that returned from MDB2::factory())
	 * @param array       $options  Options. Not required. Possible options are:
	 *                              audit_event_table_name - the table to store audit events in (defaults to 'audit_events')
	 *                              audit_changes_table_name - the table to store audit changes in (defaults to 'audit_event_changes')
	 * @return void
	 */
	public function __construct(&$dsn, $options = array())
	{
		if (is_object($dsn)) {
			if ($dsn instanceof MDB2_Driver_Common) {
				$this->_db = &$dsn;
			} else {
				throw new Exception('PHPOF2_Audit_GenericDb was passed an object which was not an MDB2_Driver_Common-derived object in the "dsn" parameter');
			}
		} else {
			$this->_db = MDB2::connect($dsn);
		}
		
		if (PEAR::isError($this->_db)) {
    		throw new Exception($this->_db->getMessage());
		}
		
		// Handle options
		if (isset($options['audit_event_table_name'])) {
			$this->_event_table_name = $options['audit_event_table_name'];
		}
		if (isset($options['audit_change_table_name'])) {
			$this->_change_table_name = $options['audit_change_table_name'];
		}
	}

	public function recordEvent($event_type, $table_name, $pri_key, $owner, $note, array $changes)
	{	
		// Mapping of input variables to the audit_events table columns
		$storage_mappings = array(
			'event_type' => 'type',
			'table_name' => 'container',
			'pri_key' => 'reference',
			'owner' => 'owner',
			'note' => 'notes',
		);

		// Quote the values ready for insertion into an SQL statement
		$properties_to_store = array();
		foreach ($storage_mappings as $var => $col_name) {
			$properties_to_store[$col_name] = $this->_db->quote($$var);
		}
		
		// Add a timestamp
		$properties_to_store['datetime'] = $this->_db->quote(date('Y-m-d H:i:s'));

		$sql = 'INSERT INTO ' . $this->_event_table_name . ' (' . implode(',', array_keys($properties_to_store)) . ') VALUES (' . implode(',', $properties_to_store) . ')';
		$this->_db->query($sql);
		$audit_event_id = $this->_db->lastInsertId($this->_event_table_name);
		
		if (count($changes) > 0){
			$id = $this->_db->quote($audit_event_id);
			$values = array();
			foreach($changes as $field => $value) {
				$values[] = '(' . $id . ','.
					$this->_db->quote($field).','.
					$this->_db->quote($value).')';
			}
			$sql = 'INSERT INTO ' . $this->_change_table_name . ' (audit_event_id, field, new_value) VALUES ' . implode(',', $values);
			$this->_db->query($sql);
		}
	}

}
